@@ -1,5 +1,5 @@
 /* GADMIN-DHCPD - An easy to use GTK+ frontend for ISC DHCPD.
- * Copyright (C) 2004 - 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2004 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #include "failover_functions.h"
 #include "get_ranges_from_current_scope.h"
 #include "show_info.h"
+#include "handle_ssh.h"
 
 extern char global_netmask[1024];
 extern char global_subnet[1024];
@@ -62,12 +63,14 @@ void apply_loadbalance_button_clicked(struct w *widgets)
     /* The apply button in the loadbalancing window */
     FILE *fp;
     long file_size;
-    char *line, *new_conf, *range_from, *range_to;
+    char *line, *new_conf, *sec_conf, *range_from, *range_to;
     int i, once = 0;
     int found_scope = 0, failover_pool_added = 0;
     int added_range_end = 0;
+    gint active_index;
     gchar *nic, *subnet, *netmask, *declaration, *range_line;
     gchar *info, *current_decl_name, *temp_decl_name;
+    gchar *sec_declaration, *settings_conf;
 
     G_CONST_RETURN gchar *decl_name;
     G_CONST_RETURN gchar *address;
@@ -156,8 +159,8 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 	line = allocate(file_size+1);
 
-	/* max entry input is 350+1024 for statics */
 	new_conf = allocate(file_size+(350*10)+1024+1);
+	sec_conf = allocate(file_size+(350*10)+1024+1);
 
 
 	/* Get the primary/secondary combo setting */
@@ -179,6 +182,25 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    "    split ", split, ";\n",
 	    "}\n\n",
 	    NULL);
+
+	    /* Create a secondary declaration suitable
+	       for upload to a remote secondary failover */
+	    sec_declaration = g_strconcat(
+	    "failover peer \"", decl_name, "\" {\n",
+	    "    secondary;\n",
+	    "    address ", remote_address, ";\n",
+	    "    port ", remote_port, ";\n",
+	    "    peer address ", address, ";\n", 
+	    "    peer port ", port, ";\n", 
+	    "    max-response-delay ", max_response_delay, ";\n", 
+	    "    max-unacked-updates ", max_unacked_updates, ";\n", 
+//	    "    load balance max seconds ", loadbalance_max_seconds, ";\n", 
+	    "    mclt ", mclt, ";\n",
+//	    "    split ", split, ";\n",
+	    "}\n\n",
+	    NULL);
+	     
+
 	}
 	else
 	{
@@ -192,12 +214,14 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    "    peer port ", remote_port, ";\n", 
 	    "    max-response-delay ", max_response_delay, ";\n", 
 	    "    max-unacked-updates ", max_unacked_updates, ";\n", 
-	    "    load balance max seconds ", loadbalance_max_seconds, ";\n", 
-/*	    "    mclt ", mclt, ";\n",     // Not in a secondary server (Works though)
-	    "    split ", split, ";\n",   // Not in a secondary server 
-*/
+//	    "    load balance max seconds ", loadbalance_max_seconds, ";\n", 
+//	    "    mclt ", mclt, ";\n",     // Not in a secondary server (Works though)
+//	    "    split ", split, ";\n",   // Not in a secondary server 
+
 	    "}\n\n",
 	    NULL);
+	    
+	    sec_declaration = g_strdup_printf("%s", "Never uploaded\n");
 	}
 	
 
@@ -213,18 +237,28 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 		/* Insert our new failover declaration */
 		strcat(new_conf, declaration);
 		strcat(new_conf, line);
+		
+		/* Insert the secondary adapted declaration */
+		strcat(sec_conf, sec_declaration);
+		strcat(sec_conf, line);
 	    }
 	    else
-	      strcat(new_conf, line);
+	    {
+	        strcat(new_conf, line);
+		strcat(sec_conf, line);
+	    }
 	}
 
 	g_free(declaration);
+	g_free(sec_declaration);
 	
 	/* Gather the rest of the configuration */
 	if( file_size > 1 )
 	while(fgets(line, file_size, fp)!=NULL)
-	  strcat(new_conf, line);	
-
+	{
+	    strcat(new_conf, line);	
+	    strcat(sec_conf, line);
+	}
 	fclose(fp);
 	free(line);
 	
@@ -241,14 +275,35 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    g_free(netmask);
 	    
 	    free(new_conf);
+	    free(sec_conf);
     	    return;
 	}
 	fputs(new_conf, fp);
 	fclose(fp);
-
 	free(new_conf);
 
 
+	/* Write the new secondary configuration in the settings directory */
+	settings_conf = g_strdup_printf("%s/%s", SETTINGS_PATH, "dhcpd.conf");
+	if((fp=fopen(settings_conf, "w+"))==NULL)
+	{
+	    info = g_strdup_printf(_("Could not write dhcpd.conf here: %s\n"), settings_conf);
+	    show_info(info);
+	    g_free(info);
+
+	    g_free(nic);
+	    g_free(subnet);
+	    g_free(netmask);
+	    
+	    free(sec_conf);
+	    g_free(settings_conf);
+    	    return;
+	}
+	fputs(sec_conf, fp);
+	fclose(fp);
+	free(sec_conf);
+	g_free(settings_conf);
+
 
 	/* Now add the ranges in this subnet to a failover pool declaration 
 	   in the same subnet declaration */
@@ -281,12 +336,14 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 	/* +1024 for the static pool addition */
 	new_conf = allocate(file_size+1024+1);
+	sec_conf = allocate(file_size+1024+1);
 
 	/* Find the correct scope and add the pool declaration */
 	if( file_size > 1 )
 	while(fgets(line, file_size, fp)!=NULL)
 	{
 	    strcat(new_conf, line);
+	    strcat(sec_conf, line);
 
 	    if( strstr(line, scope_line1) 
 	    || strstr(line, scope_line2)
@@ -299,6 +356,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 		      break;
 
 		    strcat(new_conf, line);
+		    strcat(sec_conf, line);
 		
 		    /* The interface line is found */
 		    if( strstr(line, nic_line) )
@@ -329,6 +387,8 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 	    free(line);
 	    free(new_conf);    
+	    free(sec_conf);    
+
 	    g_free(subnet);
 	    g_free(netmask);
 	    g_free(nic);
@@ -343,6 +403,10 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	"        deny dynamic bootp clients;\n",
 	NULL);
 
+	/* Secondary suitable for upload */
+	sec_declaration = g_strdup_printf("%s", declaration);
+
+
 	/* We have found the new scope, add the new failover pool
 	   before and after the scope declarations ranges */
 	if( found_scope && file_size > 1 )
@@ -355,6 +419,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    if( strstr(line, "}") )
 	    {
 		strcat(new_conf, line);
+		strcat(sec_conf, line);
 	        break;
 	    }
 
@@ -367,6 +432,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 		    /* Added failover pool */
 		    strcat(new_conf, declaration);
+		    strcat(sec_conf, sec_declaration);
 		}
 				    
 		range_from = allocate(file_size+1); // alloc too big
@@ -380,6 +446,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 		/* Add the range */		
 		strcat(new_conf, range_line);
+		strcat(sec_conf, range_line);
 	    
 		g_free(range_line);
 		free(range_from);
@@ -390,12 +457,13 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    else
 	      {
 	          strcat(new_conf, line);
+	          strcat(sec_conf, line);
 		  continue;
 	      }	
 	}
 
 
-	/* Failover pool added at first range.. if not range add end pool */
+	/* Failover pool added at first range. If not range, add end pool */
 	if( found_scope && file_size > 1 )
 	while(fgets(line, file_size, fp)!=NULL)
 	{
@@ -404,8 +472,10 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    {
 		added_range_end = 1;
 		strcat(new_conf, "    }\n");
+		strcat(sec_conf, "    }\n");
 		
 		strcat(new_conf, line);
+		strcat(sec_conf, line);
 	        break;
 	    }
 	    /* Break at end of scope */
@@ -413,6 +483,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    if( strstr(line, "}") )
 	    {
 		strcat(new_conf, line);
+		strcat(sec_conf, line);
 	        break;
 	    }
 
@@ -430,6 +501,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 		/* Add the range */		
 		strcat(new_conf, range_line);
+		strcat(sec_conf, range_line);
 	    
 		g_free(range_line);
 		free(range_from);
@@ -442,8 +514,10 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    {
 		added_range_end = 1;
 		strcat(new_conf, "    }\n");
+		strcat(sec_conf, "    }\n");
 
 		strcat(new_conf, line);
+		strcat(sec_conf, line);
 		break;
 	    }
 	}
@@ -452,8 +526,10 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	/* Gather everything else */
 	if( found_scope && file_size > 1 )
 	while(fgets(line, file_size, fp)!=NULL)
-          strcat(new_conf, line);
-
+	{
+            strcat(new_conf, line);
+            strcat(sec_conf, line);
+	}
 
 	fclose(fp);
 	free(line);
@@ -466,6 +542,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    g_free(info);
 	
 	    free(new_conf);    
+	    free(sec_conf);    
 	    g_free(subnet);
 	    g_free(netmask);
 	    g_free(nic);
@@ -480,6 +557,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    g_free(info);
 
 	    free(new_conf);
+	    free(sec_conf);
 	    g_free(subnet);
 	    g_free(netmask);
 	    g_free(nic);
@@ -489,6 +567,33 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	fclose(fp);
 	free(new_conf);
 
+	/* Write the secondary conf suitable for upload */
+	settings_conf = g_strdup_printf("%s/%s", SETTINGS_PATH, "dhcpd.conf");
+	if((fp=fopen(settings_conf, "w+"))==NULL)
+	{
+	    info = g_strdup_printf(_("Could not write dhcpd.conf here:\n%s\n"), settings_conf);
+	    show_info(info);
+	    g_free(info);
+
+	    free(sec_conf);
+	    g_free(settings_conf);
+	    g_free(subnet);
+	    g_free(netmask);
+	    g_free(nic);
+    	    return;
+	}
+	fputs(sec_conf, fp);
+	fclose(fp);
+	g_free(settings_conf);
+
+
+	/* If this is a primary loadbalancer, ask to update the secondary */
+	active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->loadbalance_combo[0]));
+	if( active_index <= 0 )
+	{
+	    show_upload_menu(widgets);
+	}
+
 	info = g_strdup_printf(_("A new loadbalancing pool was added.\n"));
 	show_info(info);
 	g_free(info);
@@ -521,8 +626,9 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
 
 
+    /* Change the current failover. The required checks have been made. */
+
 
-    /* ... Change the current failover, (The required checks have been made) ... */
 
     /* Delete the current failover declaration and insert the new one.
        Also change the declaration name in the pool. */
@@ -543,12 +649,12 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 
     line = allocate(file_size+1);
 
-    /* max entry input is 350+1024 for statics */
     new_conf = allocate(file_size+(350*10)+1024+1);
+    sec_conf = allocate(file_size+(350*10)+1024+1);
 
 
     /* Get the primary/secondary combo setting */
-    gint active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->loadbalance_combo[0]));
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->loadbalance_combo[0]));
     if( active_index <= 0 )
     {
 	/* Create a Primary failover peer declaration */
@@ -566,6 +672,23 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	"    split ", split, ";\n",
 	"}\n\n",
 	NULL);
+
+	/* Create a Secondary failover peer declaration suitable
+	   for upload to a remote loadbalancer */
+	sec_declaration = g_strconcat(
+	"failover peer \"", decl_name, "\" {\n",
+	"    secondary;\n",
+	"    address ", remote_address, ";\n",
+	"    port ", remote_port, ";\n",
+	"    peer address ", address, ";\n", 
+	"    peer port ", port, ";\n", 
+	"    max-response-delay ", max_response_delay, ";\n", 
+	"    max-unacked-updates ", max_unacked_updates, ";\n", 
+//	"    load balance max seconds ", loadbalance_max_seconds, ";\n", 
+//	"    mclt ", mclt, ";\n",
+//	"    split ", split, ";\n",
+	"}\n\n",
+	NULL);
     }
     else
     {
@@ -585,6 +708,9 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 */
         "}\n\n",
         NULL);
+
+
+	sec_declaration = g_strdup_printf("%s", "Will not be uploaded\n");
     }
 
 
@@ -609,6 +735,7 @@ void apply_loadbalance_button_clicked(struct w *widgets)
         {
 	    /* Insert our new failover declaration */
 	    strcat(new_conf, declaration);
+	    strcat(sec_conf, sec_declaration);
 
 	    /* Scroll past the old pool declaration */
 	    while(fgets(line, file_size, fp)!=NULL)
@@ -622,11 +749,19 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	    strcat(new_conf, "        failover peer \"");
 	    strcat(new_conf, decl_name);
 	    strcat(new_conf, "\";\n");
+
+	    strcat(sec_conf, "        failover peer \"");
+	    strcat(sec_conf, decl_name);
+	    strcat(sec_conf, "\";\n");
 	}
 	else
-	  strcat(new_conf, line);
+	{
+	    strcat(new_conf, line);
+	    strcat(sec_conf, line);
+	}
     }
     g_free(declaration);
+    g_free(sec_declaration);
     g_free(temp_decl_name);
 
     fclose(fp);
@@ -641,15 +776,40 @@ void apply_loadbalance_button_clicked(struct w *widgets)
 	g_free(info);
 	    
 	free(new_conf);
+	free(sec_conf);
     	return;
     }
     fputs(new_conf, fp);
     fclose(fp);
-
     free(new_conf);
 
 
-    /* Close the window */
+    /* Write the secondary conf suitable for upload */
+    settings_conf = g_strdup_printf("%s/%s", SETTINGS_PATH, "dhcpd.conf");
+    if((fp=fopen(settings_conf, "w+"))==NULL)
+    {
+	info = g_strdup_printf(_("Could not write dhcpd.conf here:\n%s\n"), settings_conf);
+	show_info(info);
+	g_free(info);
+	free(sec_conf);
+	g_free(settings_conf);
+	return;
+    }
+    fputs(sec_conf, fp);
+    fclose(fp);
+    free(sec_conf);
+    g_free(settings_conf);
+
+
+    /* If this is a primary loadbalancer then ask to update the secondary */
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->loadbalance_combo[0]));
+    if( active_index <= 0 )
+    {
+	show_upload_menu(widgets);
+    }
+
+
+    /* Close the loadbalance window */
     gtk_widget_destroy(widgets->loadbalance_window);
     
 
@@ -917,14 +1077,6 @@ void delete_loadbalance_button_clicked(struct w *widgets)
 }
 
 
-
-void cancel_loadbalance_button_clicked(struct w *widgets)
-{
-    gtk_widget_destroy(widgets->loadbalance_window);
-}
-
-
-
 void show_loadbalance_window(struct w *widgets)
 {
     GtkWidget *frame;
@@ -1028,7 +1180,7 @@ void show_loadbalance_window(struct w *widgets)
       gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_entry[0]), value);
     else
       {
-          value = g_strdup_printf(_("Primary Loadbalancer %s"), subnet);
+          value = g_strdup_printf(_("Loadbalancer %s"), subnet);
 	  utf8 = g_locale_to_utf8(value, strlen(value), NULL, NULL, NULL);
           gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_entry[0]), utf8);
           g_free(utf8);
@@ -1137,8 +1289,195 @@ void show_loadbalance_window(struct w *widgets)
     GtkWidget *cancel_loadbalance_settings_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     gtk_container_add(GTK_CONTAINER(settings_hbuttonbox), cancel_loadbalance_settings_button);
     g_signal_connect_swapped(G_OBJECT(cancel_loadbalance_settings_button), "clicked", 
-                             G_CALLBACK(cancel_loadbalance_button_clicked), widgets);
+    		G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(widgets->loadbalance_window));
 
 
     gtk_widget_show_all(widgets->loadbalance_window);
 }
+
+
+/* Upload the secondary configuration to the remote server */
+void apply_loadbalance_update_clicked(struct w *widgets)
+{
+    char **new_argv;
+    int i = 0;
+    gchar *settings_conf, *user_host_dir, *info;
+    G_CONST_RETURN gchar *remote_path;
+    G_CONST_RETURN gchar *remote_address;
+    G_CONST_RETURN gchar *remote_port;
+    G_CONST_RETURN gchar *remote_user;
+    G_CONST_RETURN gchar *remote_password;
+    G_CONST_RETURN gchar *remote_restart_cmd;
+
+    remote_path = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[0]));
+    remote_address = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[1]));
+    remote_port = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[2]));
+    remote_user = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[3]));
+    remote_password = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[4]));
+    remote_restart_cmd = gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[5]));
+
+    /* Return if any field is empty */
+    for(i=0; i<6; i++)
+    {
+	info = g_strdup_printf("%s", gtk_entry_get_text(GTK_ENTRY(widgets->loadbalance_upload_entry[i])));
+	if( info != NULL )
+	{
+	    g_free(info);
+	    continue;
+	}
+	else
+	{
+	    info = g_strdup_printf(_("All fields must be filled in.\n"));
+	    show_info(info);
+	    g_free(info);
+	    return;
+	}
+    }
+
+    settings_conf = g_strdup_printf("%s/%s", SETTINGS_PATH, "dhcpd.conf");
+
+    user_host_dir = g_strdup_printf("%s@%s:%s", remote_user, remote_address, remote_path);
+
+    /* SCP Upload the secondary conf to the remote loadbalancer */
+    new_argv = create_scp_upload_cmd((char *)remote_user, (char *)remote_port,
+                    		     (char *)settings_conf, (char *)user_host_dir);
+
+    if( ! run_ssh_cmd(new_argv, (char *)remote_password) )
+    {
+	info = g_strdup_printf(_("Failed to upload secondary configuration to the remote loadbalancer.\n"));
+	show_info(info);
+	g_free(info);
+
+	g_free(user_host_dir);
+	free(new_argv);
+	return;
+    }
+    g_free(user_host_dir);
+    free(new_argv);
+
+
+    /* SSH in to the remote server and run the dhcpd restart command on it */
+    new_argv = create_ssh_cmd((char *)remote_user, (char *)remote_address,
+			      (char *)remote_port, (char *)remote_restart_cmd);
+
+    if( ! run_ssh_cmd(new_argv, (char *)remote_password) )
+    {
+	info = g_strdup_printf(_("Failed to restart the remote dhcp server.\n"));
+	show_info(info);
+	g_free(info);
+    }
+    free(new_argv);
+
+    gtk_widget_destroy(widgets->loadbalance_upload_window);
+}
+
+
+/* We get here if a primary loadbalancer has been applied.
+   Show a question to upload the secondary-adjusted conf to the remote server. */
+void show_upload_menu(struct w *widgets)
+{
+    GtkWidget *frame;
+    GtkWidget *table;
+    GtkWidget *settings_vbox;
+    GtkTooltips *tooltips;
+    gchar *info;
+
+    /* Get the failover values for this subnet */
+    gchar *nic = g_strdup_printf("%s", global_nic);
+    gchar *subnet = g_strdup_printf("%s", global_subnet);
+    gchar *netmask = g_strdup_printf("%s", global_netmask);
+    gchar *value;
+
+    widgets->loadbalance_upload_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_position(GTK_WINDOW(widgets->loadbalance_upload_window), GTK_WIN_POS_CENTER);
+    gtk_widget_set_size_request(widgets->loadbalance_upload_window, -1, -1);
+
+    /* Set window information */
+    info = g_strdup_printf(_("GADMIN-DHCPD %s Update secondary loadbalancer ?"), VERSION);
+    gtk_window_set_title(GTK_WINDOW(widgets->loadbalance_upload_window), info);
+    g_free(info);
+
+    g_signal_connect(GTK_WINDOW(widgets->loadbalance_upload_window), "delete_event", 
+		     G_CALLBACK(gtk_widget_destroy), NULL);
+
+    settings_vbox = gtk_vbox_new(FALSE, 0);
+    gtk_container_add(GTK_CONTAINER(widgets->loadbalance_upload_window), settings_vbox);
+
+
+    tooltips = gtk_tooltips_new();
+    frame = gtk_frame_new("Upload settings:");
+
+    /* A table with 16 settings and 2 columns */
+    table = gtk_table_new(16, 2, FALSE);
+
+    gtk_box_pack_start(GTK_BOX(settings_vbox), frame, TRUE, TRUE, 1);
+    gtk_container_add(GTK_CONTAINER(frame), table);
+
+
+    /* Remote configuration path entry */
+    widgets->loadbalance_upload_entry[0] = make_entry_with_label(GTK_TABLE(table), _(" Remote configuration: "),   0,1,0,1,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[0], _("Configuration path on the remote server"), NULL);
+
+    /* Remote server address entry */
+    widgets->loadbalance_upload_entry[1] = make_entry_with_label(GTK_TABLE(table), _(" Remote server address: "),   0,1,1,2,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[1], _("The remote servers address"), NULL);
+
+    /* Remote servers ssh port entry */
+    widgets->loadbalance_upload_entry[2] = make_entry_with_label(GTK_TABLE(table), _(" Remote ssh port: "),   0,1,2,3,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[2], _("The remote servers ssh port"), NULL);
+
+    /* Remote servers ssh user entry */
+    widgets->loadbalance_upload_entry[3] = make_entry_with_label(GTK_TABLE(table), _(" Remote ssh user: "),   0,1,3,4,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[3], _("The remote servers ssh admin user"), NULL);
+
+    /* Remote servers ssh password entry */
+    widgets->loadbalance_upload_entry[4] = make_entry_with_label(GTK_TABLE(table), _(" Remote ssh password: "),   0,1,4,5,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[4], _("The remote servers ssh admin password"), NULL);
+
+    /* Remote dhcpd restart command entry */
+    widgets->loadbalance_upload_entry[5] = make_entry_with_label(GTK_TABLE(table), _(" Remote restart command: "),   0,1,5,6,350);
+    gtk_tooltips_set_tip(tooltips, widgets->loadbalance_upload_entry[5], _("The remote dhcp servers restart command"), NULL);
+
+
+
+    /* Set remote configuration path as the local one */
+    gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_upload_entry[0]), DHCPD_CONF_BUF);
+
+    /* Set remote server address (the conf addresses have been reversed) */
+    value = get_failover(nic, subnet, netmask, "peer address");
+    gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_upload_entry[1]), value);
+    g_free(value);
+    // FIX: Make it insensitive.
+
+    /* Set remote ssh port */
+    gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_upload_entry[2]), "22");
+
+    /* Set remote servers ssh user */
+    gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_upload_entry[3]), "root");
+
+    /* The password is empty */
+
+    /* Set remote servers dhcpd restart command */
+    gtk_entry_set_text(GTK_ENTRY(widgets->loadbalance_upload_entry[5]), "/etc/init.d/dhcp* restart");
+
+
+
+    /* The buttons... */
+    GtkWidget *settings_hbuttonbox = gtk_hbutton_box_new();
+    gtk_box_pack_start(GTK_BOX(settings_vbox), settings_hbuttonbox, FALSE, FALSE, 0);
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(settings_hbuttonbox), GTK_BUTTONBOX_SPREAD);
+
+    GtkWidget *apply_button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
+    gtk_container_add(GTK_CONTAINER(settings_hbuttonbox), apply_button);
+    g_signal_connect_swapped(G_OBJECT(apply_button), "clicked", 
+                             G_CALLBACK(apply_loadbalance_update_clicked), widgets);
+    
+    GtkWidget *cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+    gtk_container_add(GTK_CONTAINER(settings_hbuttonbox), cancel_button);
+
+    g_signal_connect_swapped(G_OBJECT(cancel_button), "clicked", 
+         G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(widgets->loadbalance_upload_window));
+
+
+    gtk_widget_show_all(widgets->loadbalance_upload_window);
+}
