@@ -1,5 +1,5 @@
 /* GADMIN-DHCPD - An easy to use GTK+ frontend for ISC DHCPD.
- * Copyright (C) 2004 - 2008 Magnus Loef <magnus-swe@telia.com> 
+ * Copyright (C) 2004 - 2009 Magnus Loef <magnus-swe@telia.com> 
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include "gettext.h"
 #include "widgets.h"
 #include "functions.h"
+#include "commented.h"
+#include "chars_are_digits.h"
 #include "make_settings_entries.h"
 #include "make_settings_combos.h"
 #include "populate_scope_treeview.h"
@@ -55,18 +57,13 @@ void apply_settings_button_clicked(struct w *widgets)
     /* The apply button in the settings window */
     FILE *fp;
     long file_size;
-    char *conf, *new_conf, *tmp;
-    char *ddns_update_style;
-    char *ddns_updates;
-    char *client_updates;
-    char *one_lease_per_client;
-    char *bootp;
+    char *line, *new_conf;
     gint active_index = 0;
     gchar *cmd, *settings_file, *setting;
 
     G_CONST_RETURN gchar *conf_file = gtk_entry_get_text(GTK_ENTRY(widgets->settings_entry[0]));
     G_CONST_RETURN gchar *lease_file = gtk_entry_get_text(GTK_ENTRY(widgets->settings_entry[1]));
-    
+    G_CONST_RETURN gchar *default_lease_time = gtk_entry_get_text(GTK_ENTRY(widgets->settings_entry[2]));
 
     settings_file = g_strdup_printf("%s/settings.conf", SETTINGS_PATH);
 
@@ -109,61 +106,6 @@ void apply_settings_button_clicked(struct w *widgets)
     sprintf(LEASE_FILE_BUF, "%s", lease_file);
 
 
-    /* DDNS update style */
-    ddns_update_style = allocate(1024);
-    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[0]));
-    if( active_index <= 0 )
-        snprintf(ddns_update_style, 1000, "%s", "ad-hoc");
-    else
-    if( active_index == 1 )
-	snprintf(ddns_update_style, 1000, "%s", "interim");
-    else
-	snprintf(ddns_update_style, 1000, "%s", "none");
-
-    /* DDNS updates */
-    ddns_updates = allocate(1024);
-    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[1]));
-    if( active_index <= 0 )
-        snprintf(ddns_updates, 1000, "%s", "on");
-    else
-    if( active_index == 1 )
-        snprintf(ddns_updates, 1000, "%s", "off");
-    else
-        snprintf(ddns_updates, 1000, "%s", "off");
-
-      
-    /* Client updates */    
-    client_updates = allocate(1024);
-    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[2]));
-    if( active_index <= 0 )
-        snprintf(client_updates, 1000, "%s", "allow");
-    else
-    if( active_index == 1 )
-	snprintf(client_updates, 1000, "%s", "deny");
-    else
-	snprintf(client_updates, 1000, "%s", "ignore");
-
-
-    /* One lease per client */    
-    one_lease_per_client = allocate(1024);
-    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[3]));
-    if( active_index <= 0 )
-        snprintf(one_lease_per_client, 1000, "%s", "true");
-    else
-	snprintf(one_lease_per_client, 1000, "%s", "false");
-
-    /* BOOTP */    
-    bootp = allocate(1024);
-    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[4]));
-    if( active_index <= 0 )
-        snprintf(bootp, 1000, "%s", "allow");
-    else
-    if( active_index == 1 )
-	snprintf(bootp, 1000, "%s", "deny");
-    else
-	snprintf(bootp, 1000, "%s", "ignore");
-
-    
     if( ! file_exists((char *)conf_file) )
       printf("Creating a new dhcpd.conf here: %s\n", conf_file);
 
@@ -175,57 +117,150 @@ void apply_settings_button_clicked(struct w *widgets)
     if((fp=fopen(DHCPD_CONF_BUF, "r"))==NULL)
     {
 	printf("Cant open: %s\n", DHCPD_CONF_BUF);
-	free(ddns_update_style);
-	free(ddns_updates);
-	free(client_updates);
-	free(one_lease_per_client);
-	free(bootp);
 	return;
     }
     fseek(fp, 0, SEEK_END);
     file_size = ftell(fp);
     rewind(fp);
     
-    conf = allocate(file_size+1);
+    line = allocate(file_size+1);
     new_conf = allocate(file_size+8192); /* For changes */
-    tmp = allocate(1024);
 
-    snprintf(tmp, 1000, "ddns-update-style %s;\n", ddns_update_style);
-    strcat(new_conf, tmp);
+    /* DDNS update style */
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[0]));
+    if( active_index <= 0 )
+      strcat(new_conf, "ddns-update-style ad-hoc;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "ddns-update-style interim;\n");
+    else
+      strcat(new_conf, "ddns-update-style none;\n");
 
-    snprintf(tmp, 1000, "ddns-updates %s;\n", ddns_updates);
-    strcat(new_conf, tmp);
+    /* DDNS updates */
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[1]));
+    if( active_index <= 0 )
+      strcat(new_conf, "ddns-updates on;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "ddns-updates off;\n");
+    else
+      strcat(new_conf, "ddns-updates off;\n");
+      
+    /* Client updates */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[2]));
+    if( active_index <= 0 )
+      strcat(new_conf, "allow client-updates;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "deny client-updates;\n");
+    else
+      strcat(new_conf, "ignore client-updates;\n");
 
-    snprintf(tmp, 1000, "%s client-updates;\n", client_updates);
-    strcat(new_conf, tmp);
-    
-    snprintf(tmp, 1000, "one-lease-per-client %s;\n", one_lease_per_client);
-    strcat(new_conf, tmp);
+    /* One lease per client */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[3]));
+    if( active_index <= 0 )
+      strcat(new_conf, "one-lease-per-client true;\n");
+    else
+      strcat(new_conf, "one-lease-per-client false;\n");
 
-    snprintf(tmp, 1000, "%s bootp;\n", bootp);
-    strcat(new_conf, tmp);
+    /* BOOTP */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[4]));
+    if( active_index <= 0 )
+      strcat(new_conf, "allow bootp;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "deny bootp;\n");
+    else
+      strcat(new_conf, "ignore bootp;\n");
 
-    free(tmp);
-    free(ddns_update_style);
-    free(ddns_updates);
-    free(client_updates);
-    free(one_lease_per_client);
-    free(bootp);
+    /* Booting */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[5]));
+    if( active_index <= 0 )
+      strcat(new_conf, "allow booting;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "deny booting;\n");
+    else
+      strcat(new_conf, "ignore booting;\n");
+
+    /* Unknown clients */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[6]));
+    if( active_index <= 0 )
+      strcat(new_conf, "allow unknown-clients;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "deny unknown-clients;\n");
+    else
+      strcat(new_conf, "ignore unknown-clients;\n");
+
+    /* Authoritative */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[7]));
+    if( active_index <= 0 )
+      strcat(new_conf, "authoritative;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "not authoritative;\n");
+    else
+      strcat(new_conf, "not authoritative;\n");
+
+    /* Option ip-forwarding */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[8]));
+    if( active_index <= 0 )
+      strcat(new_conf, "option ip-forwarding true;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "option ip-forwarding false;\n");
+    else
+      strcat(new_conf, "option ip-forwarding false;\n");
+
+    /* Option mask-supplier */    
+    active_index = gtk_combo_box_get_active(GTK_COMBO_BOX(widgets->settings_combo[9]));
+    if( active_index <= 0 )
+      strcat(new_conf, "option mask-supplier true;\n");
+    else
+    if( active_index == 1 )
+      strcat(new_conf, "option mask-supplier false;\n");
+    else
+      strcat(new_conf, "option mask-supplier false;\n");
+
+    /* Default-lease-time */
+    strcat(new_conf, "default-lease-time ");
+    strcat(new_conf, default_lease_time);
+    strcat(new_conf, ";\n");
 
 
     /* Get everything but the values to change */
     if( file_size > 1 )
-    while(fgets(conf, file_size, fp)!=NULL)
-    {    
-	if( ! strstr(conf, "ddns-update-style")
-	&&  ! strstr(conf, "ddns-updates")
-	&&  ! strstr(conf, "client-updates;")
-	&&  ! strstr(conf, "one-lease-per-client")
-	&&  ! strstr(conf, "bootp;") )
-	  strcat(new_conf, conf);
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	/* Subnet, host or failover declaration reached. */
+	if( ! commented(line) && strstr(line, "{") )
+	{
+	    strcat(new_conf, line);
+	    break;
+	}
+    
+	if( ! strstr(line, "ddns-update-style")
+	&&  ! strstr(line, "ddns-updates")
+	&&  ! strstr(line, "client-updates;")
+	&&  ! strstr(line, "one-lease-per-client")
+	&&  ! strstr(line, "bootp;")
+	&&  ! strstr(line, "booting;")
+	&&  ! strstr(line, "unknown-clients;")
+	&&  ! strstr(line, "authoritative;")
+	&&  ! strstr(line, "option ip-forwarding")
+	&&  ! strstr(line, "option mask-supplier")
+	&&  ! strstr(line, "default-lease-time ") )
+	  strcat(new_conf, line);
+    }
+    /* Get everything after the global options */
+    if( file_size > 1 )
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	strcat(new_conf, line);
     }
     fclose(fp);
-    free(conf);
+    free(line);
 
     /* Write the new conf */
     if((fp=fopen(DHCPD_CONF_BUF, "w+"))==NULL)
@@ -238,6 +273,7 @@ void apply_settings_button_clicked(struct w *widgets)
     fclose(fp);
 
     free(new_conf);
+
     
     /* Reread entire gui */
     populate_scope_treeview(widgets);
@@ -269,7 +305,8 @@ void show_settings_window(struct w *widgets)
 {
     FILE *fp;
     long file_size;
-    char *conf, *tmp;
+    int i=0, a=0, b=1;
+    char *line, *tmp;
     GtkWidget *frame;
     GtkWidget *table;
     GtkWidget *settings_vbox;
@@ -303,19 +340,41 @@ void show_settings_window(struct w *widgets)
     gtk_container_add(GTK_CONTAINER(frame), table);
 
     /* Max length and input 350 chars */
-    widgets->settings_entry[0] = make_entry_with_label(GTK_TABLE(table), _(" Path to dhcpd.conf: "),   0,1,0,1,350);
-    gtk_tooltips_set_tip(tooltips, widgets->settings_entry[0], "/path/to/dhcpd.conf", NULL);
+    widgets->settings_entry[0] = make_entry_with_label(GTK_TABLE(table), _(" Path to dhcpd.conf: "),   0,1,a,b,350);
+    gtk_tooltips_set_tip(tooltips, widgets->settings_entry[0], _("/path/to/dhcpd.conf"), NULL);
+    a++; b++;
 
-    widgets->settings_entry[1] = make_entry_with_label(GTK_TABLE(table), _(" Path to dhcpd.leases: "), 0,1,1,2,350);
-    gtk_tooltips_set_tip(tooltips, widgets->settings_entry[1], "/path/to/dhcpd.leases", NULL);
+    widgets->settings_entry[1] = make_entry_with_label(GTK_TABLE(table), _(" Path to dhcpd.leases: "), 0,1,a,b,350);
+    gtk_tooltips_set_tip(tooltips, widgets->settings_entry[1], _("/path/to/dhcpd.leases"), NULL);
+    a++; b++;
 
-    widgets->settings_combo[0] = make_combo_with_label(GTK_TABLE(table), _(" DDNS update style: "),    0,1,2,3,350);
-    widgets->settings_combo[1] = make_combo_with_label(GTK_TABLE(table), _(" DDNS updates: "),    0,1,3,4,350);
-    widgets->settings_combo[2] = make_combo_with_label(GTK_TABLE(table), _(" Client updates: "),       0,1,4,5,350);
-    widgets->settings_combo[3] = make_combo_with_label(GTK_TABLE(table), _(" One lease per client: "), 0,1,6,7,350);
-    widgets->settings_combo[4] = make_combo_with_label(GTK_TABLE(table), _(" BOOTP: "),                0,1,8,9,350);
+    widgets->settings_entry[2] = make_entry_with_label(GTK_TABLE(table), _(" Default lease time: "),    0,1,a,b,350);
+    gtk_tooltips_set_tip(tooltips, widgets->settings_entry[2], "604800", NULL);
+    a++; b++;
 
-    /* Set entry texts */
+    widgets->settings_combo[0] = make_combo_with_label(GTK_TABLE(table), _(" DDNS update style: "),    0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[1] = make_combo_with_label(GTK_TABLE(table), _(" DDNS updates: "),         0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[2] = make_combo_with_label(GTK_TABLE(table), _(" Client updates: "),       0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[3] = make_combo_with_label(GTK_TABLE(table), _(" One lease per client: "), 0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[4] = make_combo_with_label(GTK_TABLE(table), _(" BOOTP: "),                0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[5] = make_combo_with_label(GTK_TABLE(table), _(" Booting: "),              0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[6] = make_combo_with_label(GTK_TABLE(table), _(" Unknown clients: "),      0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[7] = make_combo_with_label(GTK_TABLE(table), _(" Authoritative: "),        0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[8] = make_combo_with_label(GTK_TABLE(table), _(" IP-forwarding: "),        0,1,a,b,350);
+    a++; b++;
+    widgets->settings_combo[9] = make_combo_with_label(GTK_TABLE(table), _(" Mask-supplier: "),        0,1,a,b,350);
+    a++; b++;
+
+
+    /* Set entry path texts */
     utf8 = g_locale_to_utf8(DHCPD_CONF_BUF, strlen(DHCPD_CONF_BUF), NULL, NULL, NULL);
     gtk_entry_set_text(GTK_ENTRY(widgets->settings_entry[0]), utf8);
 
@@ -330,13 +389,19 @@ void show_settings_window(struct w *widgets)
     fseek(fp, 0, SEEK_END);
     file_size = ftell(fp);
     rewind(fp);
-    conf = allocate(file_size+1);
+    line = allocate(file_size+1);
     tmp = allocate(file_size+1);
 
     if( file_size > 1 )
-    while(fgets(conf, file_size, fp)!=NULL)
+    while(fgets(line, file_size, fp)!=NULL)
     {    
-	if( strstr(conf, "ddns-update-style") )
+	if( commented(line) )
+	  continue;
+
+	if( strstr(line, "{") )
+	  break;
+
+	if( strstr(line, "ddns-update-style") )
 	{
 	    snprintf(tmp, 1000, "%s", _("Ad-hoc"));
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -351,7 +416,7 @@ void show_settings_window(struct w *widgets)
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[0]), utf8);
 
 	    /* Set selected. Cut 18 chars from the beginning of the line */
-	    snprintf(tmp, 1000, "%s", &conf[18]);
+	    snprintf(tmp, 1000, "%s", &line[18]);
 	    /* Cut ; and newline */
 	    tmp[strlen(tmp)-2]='\0';
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -363,7 +428,7 @@ void show_settings_window(struct w *widgets)
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[0]), 2);
 	}
 	else
-	if( strstr(conf, "ddns-updates") )
+	if( strstr(line, "ddns-updates") )
 	{
 	    snprintf(tmp, 1000, "%s", _("Yes"));
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -374,7 +439,7 @@ void show_settings_window(struct w *widgets)
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[1]), utf8);
 
 	    /* Cut 21 chars from the beginning of the line */
-	    snprintf(tmp, 1000, "%s", &conf[13]);
+	    snprintf(tmp, 1000, "%s", &line[13]);
 	    tmp[strlen(tmp)-2]='\0';
 	    if( !strcmp(tmp, "on") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[1]), 0);
@@ -382,7 +447,7 @@ void show_settings_window(struct w *widgets)
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[1]), 1);
 	}
 	else
-	if( strstr(conf, "client-updates;") )
+	if( strstr(line, "client-updates;") )
 	{
 	    snprintf(tmp, 1000, "%s", _("Allow"));
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -396,7 +461,7 @@ void show_settings_window(struct w *widgets)
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[2]), utf8);
 
-	    snprintf(tmp, 1000, "%s", conf);
+	    snprintf(tmp, 1000, "%s", line);
 	    tmp[strlen(tmp)-17]='\0';
 	    if( !strcmp(tmp, "allow") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[2]), 0);
@@ -406,7 +471,7 @@ void show_settings_window(struct w *widgets)
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[2]), 2);
 	}
 	else
-	if( strstr(conf, "one-lease-per-client") )
+	if( strstr(line, "one-lease-per-client") )
 	{
 	    snprintf(tmp, 1000, "%s", _("Yes"));
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -417,7 +482,7 @@ void show_settings_window(struct w *widgets)
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[3]), utf8);
 
 	    /* Cut 21 chars from the beginning of the line */
-	    snprintf(tmp, 1000, "%s", &conf[21]);
+	    snprintf(tmp, 1000, "%s", &line[21]);
 	    tmp[strlen(tmp)-2]='\0';
 	    if( !strcmp(tmp, "true") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[3]), 0);
@@ -425,7 +490,7 @@ void show_settings_window(struct w *widgets)
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[3]), 1);
 	}
         else
-	if( strstr(conf, "bootp;") )
+	if( strstr(line, "bootp;") )
 	{
 	    snprintf(tmp, 1000, "%s", _("Allow"));
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
@@ -439,7 +504,7 @@ void show_settings_window(struct w *widgets)
 	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
 	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[4]), utf8);
 
-	    snprintf(tmp, 1000, "%s", conf);
+	    snprintf(tmp, 1000, "%s", line);
 	    tmp[strlen(tmp)-8]='\0';
 	    if( !strcmp(tmp, "allow") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[4]), 0);
@@ -448,9 +513,114 @@ void show_settings_window(struct w *widgets)
 	    if( !strcmp(tmp, "ignore") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[4]), 2);
 	}
+        else
+	if( strstr(line, "booting;") )
+	{
+	    snprintf(tmp, 1000, "%s", _("Allow"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[5]), utf8);
+
+	    snprintf(tmp, 1000, "%s", _("Deny"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[5]), utf8);
+
+	    snprintf(tmp, 1000, "%s", line);
+	    tmp[strlen(tmp)-1]='\0';
+	    if( strstr(tmp, "allow booting;") )
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[5]), 0);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[5]), 1);
+	}
+	if( strstr(line, "unknown-clients;") )
+	{
+	    snprintf(tmp, 1000, "%s", _("Allow"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[6]), utf8);
+
+	    snprintf(tmp, 1000, "%s", _("Deny"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[6]), utf8);
+
+	    snprintf(tmp, 1000, "%s", line);
+	    tmp[strlen(tmp)-1]='\0';
+	    if( strstr(tmp, "allow") )
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[6]), 0);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[6]), 1);
+	}
+	else
+	if( strstr(line, "authoritative;") )
+	{
+	    snprintf(tmp, 1000, "%s", _("Yes"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[7]), utf8);
+
+	    snprintf(tmp, 1000, "%s", _("No"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[7]), utf8);
+
+	    snprintf(tmp, 1000, "%s", line);
+	    tmp[strlen(tmp)-1]='\0';
+	    if( ! strstr(tmp, "not ") )
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[7]), 0);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[7]), 1);
+	}
+	else
+	if( strstr(line, "option ip-forwarding") )
+	{
+	    snprintf(tmp, 1000, "%s", _("Yes"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[8]), utf8);
+
+	    snprintf(tmp, 1000, "%s", _("No"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[8]), utf8);
+
+	    snprintf(tmp, 1000, "%s", line);
+	    tmp[strlen(tmp)-1]='\0';
+	    if( strstr(tmp, "true") )
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[8]), 0);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[8]), 1);
+	}
+	else
+	if( strstr(line, "option mask-supplier") )
+	{
+	    snprintf(tmp, 1000, "%s", _("Yes"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[9]), utf8);
+
+	    snprintf(tmp, 1000, "%s", _("No"));
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->settings_combo[9]), utf8);
+
+	    snprintf(tmp, 1000, "%s", line);
+	    tmp[strlen(tmp)-1]='\0';
+	    if( strstr(tmp, "true") )
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[9]), 0);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->settings_combo[9]), 1);
+	}
+	else
+	if( strstr(line, "default-lease-time ") )
+	{
+	    /* Isolate the digits. */
+	    for(i=0; i<strlen(line)-1 && i<1000; i++)
+	    {
+	    	snprintf(tmp, 1000, "%s", &line[i]);
+		tmp[strlen(tmp)-2]='\0'; /* Remove the last ";" char. */
+		if( chars_are_digits(tmp) )
+		{
+		    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+		    gtk_entry_set_text(GTK_ENTRY(widgets->settings_entry[2]), utf8);
+		    break;
+		}
+	    }
+	}
     }
     fclose(fp);
-    free(conf);
+    free(line);
     free(tmp);
 
     if( utf8!=NULL )
